"""
Tables Extension for Python-Markdown
====================================

Added parsing of tables to Python-Markdown.

A simple example:

    First Header  | Second Header
    ------------- | -------------
    Content Cell  | Content Cell
    Content Cell  | Content Cell

Copyright 2009 - [Waylan Limberg](http://achinghead.com)
"""

from markdown import Extension
from markdown.blockprocessors import BlockProcessor
from markdown.util import etree


class TableProcessor(BlockProcessor):
    """ Process Tables. """

    def test(self, parent, block):
        rows = block.split("\n")
        return (
            len(rows) > 2
            and "|" in rows[0]
            and "|" in rows[1]
            and "-" in rows[1]
            and rows[1].strip()[0] in ["|", ":", "-"]
        )

    def run(self, parent, blocks):
        """ Parse a table block and build table. """
        block = blocks.pop(0).split("\n")
        header = block[0].strip()
        seperator = block[1].strip()
        rows = block[2:]
        # Get format type (bordered by pipes or not)
        border = False
        if header.startswith("|"):
            border = True
        # Get alignment of columns
        align = []
        for c in self._split_row(seperator, border):
            if c.startswith(":") and c.endswith(":"):
                align.append("center")
            elif c.startswith(":"):
                align.append("left")
            elif c.endswith(":"):
                align.append("right")
            else:
                align.append(None)
        # Build table
        table = etree.SubElement(parent, "table")
        table.set("class", "wftable")
        thead = etree.SubElement(table, "thead")
        self._build_row(header, thead, align, border)
        tbody = etree.SubElement(table, "tbody")
        for row in rows:
            self._build_row(row.strip(), tbody, align, border)

    def _build_row(self, row, parent, align, border):
        """ Given a row of text, build table cells. """
        tr = etree.SubElement(parent, "tr")
        tag = "td"
        if parent.tag == "thead":
            tag = "th"
        cells = self._split_row(row, border)
        # We use align here rather than cells to ensure every row
        # contains the same number of columns.
        for i, a in enumerate(align):
            c = etree.SubElement(tr, tag)
            try:
                c.text = cells[i].strip()
            except IndexError:
                c.text = ""
            if a:
                c.set("align", a)

    def _split_row(self, row, border):
        """ split a row of text into list of cells. """
        if border:
            if row.startswith("|"):
                row = row[1:]
            if row.endswith("|"):
                row = row[:-1]
        return row.split("|")


class TableExtension(Extension):
    """ Add tables to Markdown. """

    def extendMarkdown(self, md, md_globals):
        """ Add an instance of TableProcessor to BlockParser. """
        md.parser.blockprocessors.add("table", TableProcessor(md.parser), "<hashheader")


def makeExtension(configs=None):
    return TableExtension(configs=configs)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#

from django import template

from ..utils import render_markdown

register = template.Library()


@register.filter(name="markdown")
def markdown(value):
    return render_markdown(value)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# tomasz.knapik@torchbox.com 2017-12-07
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from django import forms
from django.utils.functional import cached_property

from .utils import render_markdown
from .widgets import MarkdownTextarea

try:
    from wagtail.core.blocks import TextBlock
except ImportError:
    from wagtail.wagtailcore.blocks import TextBlock


class MarkdownBlock(TextBlock):
    @cached_property
    def field(self):
        field_kwargs = {"widget": MarkdownTextarea(attrs={"rows": self.rows})}
        field_kwargs.update(self.field_options)
        return forms.CharField(**field_kwargs)

    def render_basic(self, value, context=None):
        return render_markdown(value, context)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist

from markdown.util import etree

try:  # wagtail < 2.0
    from wagtail.wagtailimages import get_image_model
except ImportError:  # wagtail >= 2.0
    from wagtail.images import get_image_model


# TODO: Default spec and class should be configurable, because they're
# dependent on how the project is set up.  Hard-coding of 'left',
# 'right' and 'full-width' should be removed.


class Linker(object):
    def run(self, fname, optstr):
        opts = {}

        opts["spec"] = "width-500"
        opts["classname"] = "left"

        for opt in optstr:
            bits = opt.split("=", 1)
            opt = bits[0]
            value = ""

            if len(bits) > 1:
                value = bits[1]

            if opt == "left":
                opts["classname"] = "left"
            elif opt == "right":
                opts["classname"] = "right"
            elif opt == "full":
                opts["classname"] = "full-width"
            elif opt == "width":
                try:
                    opts["spec"] = "width-%d" % int(value)
                except ValueError:
                    pass
        try:
            image = get_image_model().objects.get(title=fname)
        except ObjectDoesNotExist:
            return '[image "{}" not found]'.format(fname)
        except MultipleObjectsReturned:
            return '[multiple images "{}" found]'.format(fname)

        image_url = image.file.url
        rendition = image.get_rendition(opts["spec"])

        a = etree.Element("a")
        a.set("data-toggle", "lightbox")
        a.set("data-type", "image")
        a.set("href", image_url)
        img = etree.SubElement(a, "img")
        img.set("src", rendition.url)
        img.set("class", opts["classname"])
        img.set("width", str(rendition.width))
        img.set("height", str(rendition.height))
        return a



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# tomasz.knapik@torchbox.com 2017-12-07
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
try:
    from wagtail.admin.edit_handlers import FieldPanel
except ImportError:
    from wagtail.wagtailadmin.edit_handlers import FieldPanel


class MarkdownPanel(FieldPanel):
    pass



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# tomasz.knapik@torchbox.com 2017-12-07
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from django import forms
from django.conf import settings

from wagtail.utils.widgets import WidgetWithScript

try:
    from wagtail.core.telepath import register
    from wagtail.core.widget_adapters import WidgetAdapter
except ImportError:  # do-nothing fallback for Wagtail <2.13

    def register(adapter, cls):
        pass

    class WidgetAdapter:
        pass


class MarkdownTextarea(WidgetWithScript, forms.widgets.Textarea):
    def render_js_init(self, id_, name, value):
        autodownload_fontawesome = getattr(
            settings, "WAGTAILMARKDOWN_AUTODOWNLOAD_FONTAWESOME", None
        )
        if autodownload_fontawesome is not None:
            autodownload = "true" if autodownload_fontawesome else "false"
            return 'easymdeAttach("{0}", {1});'.format(id_, autodownload)
        return 'easymdeAttach("{0}");'.format(id_)

    @property
    def media(self):
        return forms.Media(
            css={
                "all": (
                    "wagtailmarkdown/css/easymde.min.css",
                    "wagtailmarkdown/css/easymde.tweaks.css",
                )
            },
            js=(
                "wagtailmarkdown/js/easymde.min.js",
                "wagtailmarkdown/js/easymde.attach.js",
            ),
        )


class MarkdownTextareaAdapter(WidgetAdapter):
    js_constructor = "wagtailmarkdown.widgets.MarkdownTextarea"

    class Media:
        js = ["wagtailmarkdown/js/markdown-textarea-adapter.js"]


register(MarkdownTextareaAdapter(), MarkdownTextarea)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from django.conf import settings
from django.utils.encoding import smart_text
from django.utils.safestring import mark_safe

import bleach
import markdown

from .mdx import linker, tables


def render_markdown(text, context=None):
    """
    Turn markdown into HTML.
    """
    if context is None or not isinstance(context, dict):
        context = {}
    markdown_html = _transform_markdown_into_html(text)
    sanitised_markdown_html = _sanitise_markdown_html(markdown_html)
    return mark_safe(sanitised_markdown_html)


def _transform_markdown_into_html(text):
    return markdown.markdown(smart_text(text), **_get_markdown_kwargs())


def _sanitise_markdown_html(markdown_html):
    return bleach.clean(markdown_html, **_get_bleach_kwargs())


def _get_bleach_kwargs():
    bleach_kwargs = {}
    bleach_kwargs["tags"] = [
        "p",
        "div",
        "span",
        "h1",
        "h2",
        "h3",
        "h4",
        "h5",
        "h6",
        "tt",
        "pre",
        "em",
        "strong",
        "ul",
        "sup",
        "li",
        "dl",
        "dd",
        "dt",
        "code",
        "img",
        "a",
        "table",
        "tr",
        "th",
        "td",
        "tbody",
        "caption",
        "colgroup",
        "thead",
        "tfoot",
        "blockquote",
        "ol",
        "hr",
        "br",
    ]
    bleach_kwargs["attributes"] = {
        "*": [
            "class",
            "style",
            "id",
        ],
        "a": [
            "href",
            "target",
            "rel",
        ],
        "img": [
            "src",
            "alt",
        ],
        "tr": [
            "rowspan",
            "colspan",
        ],
        "td": [
            "rowspan",
            "colspan",
            "align",
        ],
    }
    bleach_kwargs["styles"] = [
        "color",
        "background-color",
        "font-family",
        "font-weight",
        "font-size",
        "width",
        "height",
        "text-align",
        "border",
        "border-top",
        "border-bottom",
        "border-left",
        "border-right",
        "padding",
        "padding-top",
        "padding-bottom",
        "padding-left",
        "padding-right",
        "margin",
        "margin-top",
        "margin-bottom",
        "margin-left",
        "margin-right",
    ]
    return bleach_kwargs


def _get_markdown_kwargs():
    markdown_kwargs = {}
    markdown_kwargs["extensions"] = [
        "extra",
        "codehilite",
        tables.TableExtension(),
        linker.LinkerExtension(
            {
                "__default__": "wagtailmarkdown.mdx.linkers.page",
                "page:": "wagtailmarkdown.mdx.linkers.page",
                "image:": "wagtailmarkdown.mdx.linkers.image",
                "doc:": "wagtailmarkdown.mdx.linkers.document",
            }
        ),
    ]

    if hasattr(settings, "WAGTAILMARKDOWN_EXTENSIONS"):
        markdown_kwargs["extensions"] += settings.WAGTAILMARKDOWN_EXTENSIONS

    markdown_kwargs["extension_configs"] = {
        "codehilite": [
            ("guess_lang", False),
        ]
    }

    if hasattr(settings, "WAGTAILMARKDOWN_EXTENSIONS_CONFIG"):
        markdown_kwargs["extension_configs"].update(
            settings.WAGTAILMARKDOWN_EXTENSIONS_CONFIG
        )

    markdown_kwargs["output_format"] = "html5"
    return markdown_kwargs



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from importlib import import_module

import markdown

LINKER_RE = r"<:([a-z]+:)?([^>|\n]+)((\|[^>|\n]+){0,})>"


class LinkerPattern(markdown.inlinepatterns.Pattern):
    def __init__(self, re, md, linktypes):
        markdown.inlinepatterns.Pattern.__init__(self, re, md)
        self.linktypes = linktypes

    def handleMatch(self, m):
        linktypes = self.linktypes
        opts = []
        if m.group(3) is not None and len(m.group(4)):
            opts = m.group(4).split("|")[1:]

        type = m.group(2)
        if type is None:
            type = "__default__"
        mod = import_module(linktypes[type])
        c = mod.Linker()

        return c.run(m.group(3), opts)
        return "[invalid link]"


class LinkerExtension(markdown.Extension):
    def __init__(self, linktypes):
        markdown.Extension.__init__(self)
        self.linktypes = linktypes

    def extendMarkdown(self, md, md_globals):
        md.inlinePatterns["linker"] = LinkerPattern(LINKER_RE, md, self.linktypes)


def makeExtension(configs=None):
    return LinkerExtension(configs=configs)



#! /usr/bin/env/python
# vim:sw=4 ts=4 et:
#
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#

import subprocess

from setuptools import find_packages, setup


def get_git_revision_hash():
    try:
        git_hash = subprocess.check_output(["git", "rev-parse", "HEAD"])
    except subprocess.CalledProcessError:
        return "main"
    else:
        return git_hash.decode("ascii").splitlines()[0]


README = "https://github.com/torchbox/wagtail-markdown/blob/{hash}/README.md"
README = README.format(hash=get_git_revision_hash())


INSTALL_REQUIRES = [
    "Markdown>=3,<4",
    "bleach>=3.3.0,<4",
    "Wagtail>=2.0",
]


TESTING_REQUIRES = [
    "dj_database_url==0.5.0",
]


CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Web Environment",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: zlib/libpng License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Framework :: Django",
    "Framework :: Django :: 2.2",
    "Framework :: Django :: 3.0",
    "Framework :: Django :: 3.1",
    "Framework :: Wagtail",
    "Framework :: Wagtail :: 2",
]


setup(
    name="wagtail-markdown",
    version="0.7.0-rc2",
    description="Markdown support for Wagtail",
    long_description="Provides Markdown page field and streamfield block for "
    "Wagtail. More info: {}".format(README),
    author="Felicity Tarnell",
    author_email="felicity@torchbox.com",
    url="https://github.com/torchbox/wagtail-markdown",
    project_urls={
        "Changelog": "https://github.com/torchbox/wagtail-markdown/blob/main/CHANGELOG.md",
    },
    install_requires=INSTALL_REQUIRES,
    license="zlib",
    packages=find_packages(),
    include_package_data=True,
    classifiers=CLASSIFIERS,
    extras_require={"testing": TESTING_REQUIRES},
)
